@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2011-2015  Leo Singer
+# Copyright (C) 2011-2017  Leo Singer
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,23 +18,23 @@
 Overplot contours for a large number of sky maps, in geographical coordinates.
 This can reveal patterns due to the priors (i.e., the network antenna pattern).
 """
-__author__ = "Leo Singer <leo.singer@ligo.org>"
 
 
 # Command line interface
 
 from lalinference.bayestar import command
 parser = command.ArgumentParser(parents=[command.figure_parser])
-parser.add_argument('--contour', metavar='PERCENT', type=float, default=90,
+parser.add_argument(
+    '--contour', metavar='PERCENT', type=float, default=90,
     help='plot contour enclosing this percentage of'
     ' probability mass [default: %(default)s]')
-parser.add_argument('--alpha', metavar='ALPHA', type=float, default=0.1,
-    help='alpha blending for each sky map [default: %(default)s]')
-parser.add_argument('--transparent', action='store_true', default=False,
-    help='Save image with transparent background [default: %(default)s]')
 parser.add_argument(
-    'fitsfileglobs', metavar='GLOB.fits[.gz]', nargs='+',
+    '--alpha', metavar='ALPHA', type=float, default=0.1,
+    help='alpha blending for each sky map [default: %(default)s]')
+parser.add_argument(
+    'fitsfilenames', metavar='GLOB.fits[.gz]', nargs='+', action='glob',
     help='Input FITS filenames and/or globs')
+parser.set_defaults(colormap=None)
 opts = parser.parse_args()
 
 # Late imports
@@ -55,14 +55,17 @@ ax.grid()
 
 progress = ProgressBar()
 
-progress.update(-1, 'obtaining filenames of sky maps')
-fitsfilenames = tuple(command.chainglob(opts.fitsfileglobs))
-
-progress.max = len(fitsfilenames)
+progress.max = len(opts.fitsfilenames)
 
 matplotlib.rc('path', simplify=True, simplify_threshold=1)
 
-for count_records, fitsfilename in enumerate(fitsfilenames):
+if opts.colormap is None:
+    colors = ['k'] * len(opts.fitsfilenames)
+else:
+    colors = matplotlib.cm.get_cmap(opts.colormap)
+    colors = colors(np.linspace(0, 1, len(opts.fitsfilenames)))
+for count_records, (color, fitsfilename) in enumerate(
+        zip(colors, opts.fitsfilenames)):
     progress.update(count_records, fitsfilename)
     skymap, metadata = fits.read_sky_map(fitsfilename, nest=None)
     nside = hp.npix2nside(len(skymap))
@@ -72,18 +75,13 @@ for count_records, fitsfilename in enumerate(fitsfilenames):
     region = np.empty(skymap.shape)
     region[indices] = 100 * np.cumsum(skymap[indices])
     plot.healpix_contour(
-        region, nest=metadata['nest'], dlon=-gmst, colors='k', linewidths=0.5,
-        levels=[opts.contour], alpha=opts.alpha)
+        region, nest=metadata['nest'], dlon=-gmst,
+        colors=[color], linewidths=0.5, levels=[opts.contour],
+        alpha=opts.alpha)
 
 progress.update(-1, 'saving figure')
 
-# If we are using a new enough version of matplotlib, then
-# add a white outline to all text to make it stand out from the background.
+# Add a white outline to all text to make it stand out from the background.
 plot.outline_text(ax)
 
-if opts.transparent:
-    fig.patch.set_alpha(0.)
-    ax.patch.set_alpha(0.)
-    ax.set_alpha(0.)
-
 opts.output()
