@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 #
 # Copyright (C) 2013-2017  Leo Singer
 #
@@ -19,18 +20,155 @@
 Postprocessing utilities for HEALPix sky maps
 """
 from __future__ import division
-
-
-import numpy as np
-import healpy as hp
+from __future__ import print_function
 import collections
-import lal
-import lalsimulation
+import pkg_resources
+
+from astropy import constants
+from astropy.coordinates import (CartesianRepresentation, SkyCoord,
+                                 UnitSphericalRepresentation)
+from astropy import units as u
+from astropy.wcs import WCS
+import healpy as hp
+import numpy as np
 from scipy.interpolate import interp1d
-from . import distance
-from . import moc
+import six
+
+from .. import distance
+from .. import moc
 from ..healpix_tree import *
 
+try:
+    pkg_resources.require('numpy >= 1.10.0')
+except pkg_resources.VersionConflict:
+    """FIXME: in numpy < 1.10.0, the digitize() function only works on 1D
+    arrays. Remove this workaround once we require numpy >= 1.10.0.
+
+    Example:
+    >>> digitize([3], np.arange(5))
+    array([4])
+    >>> digitize(3, np.arange(5))
+    array(4)
+    >>> digitize([[3]], np.arange(5))
+    array([[4]])
+    >>> digitize([], np.arange(5))
+    array([], dtype=int64)
+    >>> digitize([[], []], np.arange(5))
+    array([], shape=(2, 0), dtype=int64)
+    """
+    def digitize(x, *args, **kwargs):
+        x_flat = np.ravel(x)
+        x_shape = np.shape(x)
+        if len(x_flat) == 0:
+            return np.zeros(x_shape, dtype=np.intp)
+        else:
+            return np.digitize(x_flat, *args, **kwargs).reshape(x_shape)
+
+    # FIXME: np.cov() got the aweights argument in version 1.10.
+    # Until we require numpy >= 1.10, this is copied from the Numpy source.
+    def cov(m, y=None, rowvar=True, bias=False, ddof=None, fweights=None,
+            aweights=None):
+        from numpy import array, average, dot, sum
+        import warnings
+
+        # Check inputs
+        if ddof is not None and ddof != int(ddof):
+            raise ValueError(
+                "ddof must be integer")
+
+        # Handles complex arrays too
+        m = np.asarray(m)
+        if m.ndim > 2:
+            raise ValueError("m has more than 2 dimensions")
+
+        if y is None:
+            dtype = np.result_type(m, np.float64)
+        else:
+            y = np.asarray(y)
+            if y.ndim > 2:
+                raise ValueError("y has more than 2 dimensions")
+            dtype = np.result_type(m, y, np.float64)
+
+        X = array(m, ndmin=2, dtype=dtype)
+        if not rowvar and X.shape[0] != 1:
+            X = X.T
+        if X.shape[0] == 0:
+            return np.array([]).reshape(0, 0)
+        if y is not None:
+            y = array(y, copy=False, ndmin=2, dtype=dtype)
+            if not rowvar and y.shape[0] != 1:
+                y = y.T
+            X = np.concatenate((X, y), axis=0)
+
+        if ddof is None:
+            if bias == 0:
+                ddof = 1
+            else:
+                ddof = 0
+
+        # Get the product of frequencies and weights
+        w = None
+        if fweights is not None:
+            fweights = np.asarray(fweights, dtype=float)
+            if not np.all(fweights == np.around(fweights)):
+                raise TypeError(
+                    "fweights must be integer")
+            if fweights.ndim > 1:
+                raise RuntimeError(
+                    "cannot handle multidimensional fweights")
+            if fweights.shape[0] != X.shape[1]:
+                raise RuntimeError(
+                    "incompatible numbers of samples and fweights")
+            if any(fweights < 0):
+                raise ValueError(
+                    "fweights cannot be negative")
+            w = fweights
+        if aweights is not None:
+            aweights = np.asarray(aweights, dtype=float)
+            if aweights.ndim > 1:
+                raise RuntimeError(
+                    "cannot handle multidimensional aweights")
+            if aweights.shape[0] != X.shape[1]:
+                raise RuntimeError(
+                    "incompatible numbers of samples and aweights")
+            if any(aweights < 0):
+                raise ValueError(
+                    "aweights cannot be negative")
+            if w is None:
+                w = aweights
+            else:
+                w *= aweights
+
+        avg, w_sum = average(X, axis=1, weights=w, returned=True)
+        w_sum = w_sum[0]
+
+        # Determine the normalization
+        if w is None:
+            fact = X.shape[1] - ddof
+        elif ddof == 0:
+            fact = w_sum
+        elif aweights is None:
+            fact = w_sum - ddof
+        else:
+            fact = w_sum - ddof*sum(w*aweights)/w_sum
+
+        if fact <= 0:
+            warnings.warn("Degrees of freedom <= 0 for slice",
+                          RuntimeWarning, stacklevel=2)
+            fact = 0.0
+
+        X -= avg[:, None]
+        if w is None:
+            X_T = X.T
+        else:
+            X_T = (X*w).T
+        c = dot(X, X_T.conj())
+        c *= 1. / np.float64(fact)
+        return c.squeeze()
+else:
+    cov = np.cov
+    digitize = np.digitize
+
 
 def flood_fill(nside, ipix, m, nest=False):
     """Stack-based flood fill algorithm in HEALPix coordinates.
@@ -143,7 +281,7 @@ def find_injection_moc(sky_map, true_ra=None, true_dec=None, true_dist=None,
         true_phi = true_ra
         true_pix = hp.ang2pix(max_nside, true_theta, true_phi, nest=True)
         i = np.argsort(max_ipix)
-        true_idx = i[np.digitize(true_pix, max_ipix[i]) - 1]
+        true_idx = i[digitize(true_pix, max_ipix[i]) - 1]
 
     # Find the angular offset between the mode and true locations.
     mode_theta, mode_phi = hp.pix2ang(
@@ -184,7 +322,7 @@ def find_injection_moc(sky_map, true_ra=None, true_dec=None, true_dist=None,
         searched_prob = prob[true_idx]
 
     # Find the contours of the given credible levels.
-    contour_idxs = np.digitize(contours, prob) - 1
+    contour_idxs = digitize(contours, prob) - 1
 
     # For each of the given confidence levels, compute the area of the
     # smallest region containing that probability.
@@ -232,7 +370,9 @@ def find_injection_moc(sky_map, true_ra=None, true_dec=None, true_dist=None,
 
         # Set up distance grid.
         n_r = 1000
-        max_r = max(6 * distmean, np.max(true_dist))
+        max_r = 6 * distmean
+        if true_dist is not None and true_dist > max_r:
+            max_r = true_dist
         d_r = max_r / n_r
         r = d_r * np.arange(1, n_r)
 
@@ -267,7 +407,7 @@ def find_injection_moc(sky_map, true_ra=None, true_dec=None, true_dist=None,
             searched_vol = searched_prob_vol = np.nan
         else:
             i_radec = true_idx
-            i_dist = np.digitize(true_dist, r) - 1
+            i_dist = digitize(true_dist, r) - 1
             searched_prob_vol = P[i_radec, i_dist]
             searched_vol = V[i_radec, i_dist]
     else:
@@ -349,7 +489,7 @@ def contour(m, levels, nest=False, degrees=False, simplify=True):
         # Find credible region
         indicator = (m >= level)
 
-        # Construct a graph of the eges of the contour.
+        # Construct a graph of the edges of the contour.
         graph = nx.Graph()
         face_pairs = set()
         for ipix1, ipix2 in enumerate(neighbors):
@@ -428,61 +568,27 @@ def find_greedy_credible_levels(p, ranking=None):
     return cls.reshape(p.shape)
 
 
+def _get_detector_location(ifo):
+    if isinstance(ifo, six.string_types):
+        try:
+            import lalsimulation
+        except ImportError:
+            raise RuntimeError('Looking up detectors by name '
+                               'requires the lalsimulation package.')
+        ifo = lalsimulation.DetectorPrefixToLALDetector(ifo)
+    try:
+        ifo = ifo.location
+    except AttributeError:
+        pass
+    return ifo
+
+
 def get_detector_pair_axis(ifo1, ifo2, gmst):
-    """Find the sky position where the line between two detectors pierces the
-    celestial sphere.
-
-    Parameters
-    ----------
-
-    ifo1 : str or `~lal.Detector` or `~np.ndarray`
-        The first detector; either the name of the detector (e.g. `'H1'`), or a
-        `lal.Detector` object (e.g., as returned by
-        `lalsimulation.DetectorPrefixToLALDetector('H1')` or the geocentric
-        Cartesian position of the detection in meters.
-
-    ifo2 : str or `~lal.Detector` or `~np.ndarray`
-        The second detector; same as described above.
-
-    gmst : float
-        The Greenwich mean sidereal time in radians, as returned by
-        `lal.GreenwichMeanSiderealTime`.
-
-    Returns
-    -------
-
-    pole_ra : float
-        The right ascension in radians at which a ray from `ifo1` to `ifo2`
-        would pierce the celestial sphere.
-
-    pole_dec : float
-        The declination in radians at which a ray from `ifo1` to `ifo2` would
-        pierce the celestial sphere.
-
-    light_travel_time : float
-        The light travel time from `ifo1` to `ifo2` in seconds.
-    """
-
-    # Get location of detectors if ifo1, ifo2 are LAL detector structs
-    try:
-        ifo1 = lalsimulation.DetectorPrefixToLALDetector(ifo1)
-    except TypeError:
-        pass
-    try:
-        ifo1 = ifo1.location
-    except AttributeError:
-        pass
-    try:
-        ifo2 = lalsimulation.DetectorPrefixToLALDetector(ifo2)
-    except TypeError:
-        pass
-    try:
-        ifo2 = ifo2.location
-    except AttributeError:
-        pass
+    ifo1 = _get_detector_location(ifo1)
+    ifo2 = _get_detector_location(ifo2)
 
     n = ifo2 - ifo1
-    light_travel_time = np.sqrt(np.sum(np.square(n))) / lal.C_SI
+    light_travel_time = np.sqrt(np.sum(np.square(n))) / constants.c.value
     (theta,), (phi,) = hp.vec2ang(n)
     pole_ra = (gmst + phi) % (2 * np.pi)
     pole_dec = 0.5 * np.pi - theta
@@ -615,3 +721,257 @@ def smooth_ud_grade(m, nside, nest=False):
     theta, phi = hp.pix2ang(nside, np.arange(npix), nest=nest)
     new_m = hp.get_interp_val(m, theta, phi, nest=nest)
     return new_m * len(m) / len(new_m)
+
+
+def posterior_mean(prob, nest=False):
+    npix = len(prob)
+    nside = hp.npix2nside(npix)
+    xyz = hp.pix2vec(nside, np.arange(npix), nest=nest)
+    mean_xyz = np.average(xyz, axis=1, weights=prob)
+    pos = SkyCoord(*mean_xyz, representation=CartesianRepresentation)
+    pos.representation = UnitSphericalRepresentation
+    return pos
+
+
+def posterior_max(prob, nest=False):
+    npix = len(prob)
+    nside = hp.npix2nside(npix)
+    i = np.argmax(prob)
+    return SkyCoord(
+        *hp.pix2ang(nside, i, nest=nest, lonlat=True), unit=u.deg)
+
+
+def find_ellipse(prob, cl=90, projection='ARC', nest=False):
+    """For a HEALPix map, find an ellipse that contains a given probability.
+
+    Parameters
+    ----------
+    prob : np.ndarray
+        The HEALPix probability map.
+    cl : float
+        The desired credible level (default: 90).
+    projection : str
+        The WCS projection (default: 'ARC', or zenithal equidistant).
+        For a list of possible values, see:
+        http://docs.astropy.org/en/stable/wcs/index.html#supported-projections
+    nest : bool
+        HEALPix pixel ordering (default: False, or ring ordering).
+
+    Returns
+    -------
+    ra : float
+        The ellipse center right ascension in degrees.
+    dec : float
+        The ellipse center right ascension in degrees.
+    pa : float
+        The position angle of the semimajor axis in degrees.
+    a : float
+        The lenth of the semimajor axis in degrees.
+    b : float
+        The length o the semiminor axis in degrees.
+
+    Examples
+    --------
+
+    I'm not showing the `ra` or `pa` output from the examples below because
+    the right ascension is arbitary when dec=90° and the position angle is
+    arbitrary when a=b; their arbitrary values may vary depending on your math
+    library. Also, I add 0.0 to the outputs because on some platforms you tend
+    to get values of dec or pa that get rounded to -0.0, which is within
+    numerical precision but would break the doctests (see
+    https://stackoverflow.com/questions/11010683).
+
+    Example 1
+    ~~~~~~~~~
+
+    This is an example sky map that is uniform in sin(theta) out to a given
+    radius in degrees. The 90% credible radius should be 0.9 * radius. (There
+    will be deviations for small radius due to finite resolution.)
+
+    >>> def make_uniform_in_sin_theta(radius, nside=512):
+    ...     npix = hp.nside2npix(nside)
+    ...     theta, phi = hp.pix2ang(nside, np.arange(npix))
+    ...     theta_max = np.deg2rad(radius)
+    ...     prob = np.where(theta <= theta_max, 1 / np.sin(theta), 0)
+    ...     return prob / prob.sum()
+    ...
+
+    >>> prob = make_uniform_in_sin_theta(1)
+    >>> ra, dec, pa, a, b = np.around(find_ellipse(prob), 5) + 0
+    >>> print(dec, a, b)
+    90.0 0.82241 0.82241
+
+    >>> prob = make_uniform_in_sin_theta(10)
+    >>> ra, dec, pa, a, b = np.around(find_ellipse(prob), 5) + 0
+    >>> print(dec, a, b)
+    90.0 9.05512 9.05512
+
+    >>> prob = make_uniform_in_sin_theta(120)
+    >>> ra, dec, pa, a, b = np.around(find_ellipse(prob), 5) + 0
+    >>> print(dec, a, b)
+    90.0 107.9745 107.9745
+
+    Example 2
+    ~~~~~~~~~
+
+    These are approximately Gaussian distributions.
+
+    >>> from scipy import stats
+    >>> def make_gaussian(mean, cov, nside=512):
+    ...     npix = hp.nside2npix(nside)
+    ...     xyz = np.transpose(hp.pix2vec(nside, np.arange(npix)))
+    ...     # FIXME: stats.multivariate_normal was added in scipy 0.14,
+    ...     # but we still need to support an older version on our
+    ...     # Scientific Linux 7 clusters.
+    ...     #
+    ...     # dist = stats.multivariate_normal(mean, cov)
+    ...     # prob = dist.pdf(xyz)
+    ...     if np.ndim(cov) == 0:
+    ...         cov = [cov] * 3
+    ...     if np.ndim(cov) == 1:
+    ...         cov = np.diag(cov)
+    ...     d = xyz - mean
+    ...     prob = np.exp(-0.5 * (np.linalg.solve(cov, d.T) * d.T).sum(0))
+    ...     return prob / prob.sum()
+    ...
+
+    This one is centered at RA=45°, Dec=0° and has a standard deviation of ~1°.
+
+    >>> prob = make_gaussian(
+    ...     [1/np.sqrt(2), 1/np.sqrt(2), 0],
+    ...     np.square(np.deg2rad(1)))
+    ...
+    >>> ra, dec, pa, a, b = np.around(find_ellipse(prob), 5) + 0
+    >>> print(ra, dec, a, b)
+    45.0 0.0 2.14209 2.14209
+
+    This one is centered at RA=45°, Dec=0°, and is elongated in the north-south
+    direction.
+
+    >>> prob = make_gaussian(
+    ...     [1/np.sqrt(2), 1/np.sqrt(2), 0],
+    ...     np.diag(np.square(np.deg2rad([1, 1, 10]))))
+    ...
+    >>> ra, dec, pa, a, b = np.around(find_ellipse(prob), 5) + 0
+    >>> print(ra, dec, pa, a, b)
+    45.0 0.0 0.0 13.44746 2.1082
+
+    This one is centered at RA=0°, Dec=0°, and is elongated in the east-west
+    direction.
+
+    >>> prob = make_gaussian(
+    ...     [1, 0, 0],
+    ...     np.diag(np.square(np.deg2rad([1, 10, 1]))))
+    ...
+    >>> ra, dec, pa, a, b = np.around(find_ellipse(prob), 5) + 0
+    >>> print(dec, pa, a, b)
+    0.0 90.0 13.4194 2.1038
+
+    This one is centered at RA=0°, Dec=0°, and is tilted about 10° to the west
+    of north.
+
+    >>> prob = make_gaussian(
+    ...     [1, 0, 0],
+    ...     [[0.1, 0, 0],
+    ...      [0, 0.1, -0.15],
+    ...      [0, -0.15, 1]])
+    ...
+    >>> ra, dec, pa, a, b = np.around(find_ellipse(prob), 5) + 0
+    >>> print(dec, pa, a, b)
+    0.0 170.78253 63.82809 34.00824
+
+    This one is centered at RA=0°, Dec=0°, and is tilted about 10° to the east
+    of north.
+
+    >>> prob = make_gaussian(
+    ...     [1, 0, 0],
+    ...     [[0.1, 0, 0],
+    ...      [0, 0.1, 0.15],
+    ...      [0, 0.15, 1]])
+    ...
+    >>> ra, dec, pa, a, b = np.around(find_ellipse(prob), 5) + 0
+    >>> print(dec, pa, a, b)
+    0.0 9.21747 63.82809 34.00824
+
+    This one is centered at RA=0°, Dec=0°, and is tilted about 80° to the east
+    of north.
+
+    >>> prob = make_gaussian(
+    ...     [1, 0, 0],
+    ...     [[0.1, 0, 0],
+    ...      [0, 1, 0.15],
+    ...      [0, 0.15, 0.1]])
+    ...
+    >>> ra, dec, pa, a, b = np.around(find_ellipse(prob), 5) + 0
+    >>> print(dec, pa, a, b)
+    0.0 80.78252 63.82533 34.00677
+
+    This one is centered at RA=0°, Dec=0°, and is tilted about 80° to the west
+    of north.
+
+    >>> prob = make_gaussian(
+    ...     [1, 0, 0],
+    ...     [[0.1, 0, 0],
+    ...      [0, 1, -0.15],
+    ...      [0, -0.15, 0.1]])
+    ...
+    >>> ra, dec, pa, a, b = np.around(find_ellipse(prob), 5) + 0
+    >>> print(dec, pa, a, b)
+    0.0 99.21748 63.82533 34.00677
+    """
+    npix = len(prob)
+    nside = hp.npix2nside(npix)
+
+    # Find mean right ascension and declination.
+    xyz0 = (hp.pix2vec(nside, np.arange(npix), nest=nest) * prob).sum(axis=1)
+    (ra,), (dec,) = hp.vec2ang(xyz0, lonlat=True)
+
+    # Construct WCS with the specified projection
+    # and centered on mean direction.
+    w = WCS()
+    w.wcs.crval = [ra, dec]
+    w.wcs.ctype = ['RA---' + projection, 'DEC--' + projection]
+
+    # Transform HEALPix to zenithal equidistant coordinates.
+    xy = w.wcs_world2pix(
+        np.transpose(
+            hp.pix2ang(
+                nside, np.arange(npix), nest=nest, lonlat=True)), 1)
+
+    # Keep only values that were inside the projection.
+    keep = np.logical_and.reduce(np.isfinite(xy), axis=1)
+    xy = xy[keep]
+    prob = prob[keep]
+
+    # Find covariance matrix.
+    c = cov(xy, aweights=prob, rowvar=False)
+
+    # If each point is n-sigma from the center, find n.
+    nsigmas = np.sqrt(np.sum(xy.T * np.linalg.solve(c, xy.T), axis=0))
+
+    # Find the number of sigma that enclose the cl% credible level.
+    i = np.argsort(nsigmas)
+    nsigmas = nsigmas[i]
+    cls = np.cumsum(prob[i])
+    nsigma = np.interp(1e-2 * cl, cls, nsigmas)
+
+    # If the credible level is not within the projection,
+    # then stop here and return all nans.
+    if 1e-2 * cl > cls[-1]:
+        return np.nan, np.nan, np.nan, np.nan, np.nan
+
+    # Find the eigendecomposition of the covariance matrix.
+    w, v = np.linalg.eigh(c)
+
+    # Find the semi-minor and semi-major axes.
+    b, a = nsigma * np.sqrt(w)
+
+    # Find the position angle.
+    pa = np.rad2deg(np.arctan2(*v[:, 1]))
+
+    # An ellipse is symmetric under rotations of 180°.
+    # Return the smallest possible positive position angle.
+    pa %= 180
+
+    # Done!
+    return ra, dec, pa, a, b
