@@ -68,7 +68,8 @@ import time
 import lal
 import six
 from astropy.table import Table
-from ..bayestar import moc
+from .. import moc
+from ..util.file import rm_f
 
 # FIXME: Remove this after all Astropy monkeypatches are obsolete.
 import astropy
@@ -187,12 +188,12 @@ def identity(x):
 
 def instruments_to_fits(value):
     if not isinstance(value, six.string_types):
-        value = str(lsctables.ifos_from_instrument_set(value))
+        value = str(lsctables.instrumentsproperty.set(value))
     return value
 
 
 def instruments_from_fits(value):
-    return {str(ifo) for ifo in lsctables.instrument_set_from_ifos(value)}
+    return {str(ifo) for ifo in lsctables.instrumentsproperty.get(value)}
 
 
 DEFAULT_NUNIQ_NAMES = ('PROBDENSITY', 'DISTMU', 'DISTSIGMA', 'DISTNORM')
@@ -201,7 +202,7 @@ DEFAULT_NESTED_NAMES = ('PROB', 'DISTMU', 'DISTSIGMA', 'DISTNORM')
 DEFAULT_NESTED_UNITS = (u.pix**-1, u.Mpc, u.Mpc, u.Mpc**-2)
 FITS_META_MAPPING = (
     ('objid', 'OBJECT', 'Unique identifier for this event',
-     identity, identity),
+     str, identity),
     ('url', 'REFERENC', 'URL of this event', identity, identity),
     ('instruments', 'INSTRUME', 'Instruments that triggered this event',
      instruments_to_fits, instruments_from_fits),
@@ -409,7 +410,6 @@ def write_sky_map(filename, m, **kwargs):
         #
         # Also this works around https://github.com/astropy/astropy/pull/5720,
         # which was fixed in astropy 1.3.1.
-        from ..bayestar.command import rm_f
         rm_f(filename)
         m.write(filename, format='fits')
 
@@ -419,7 +419,7 @@ def write_sky_map(filename, m, **kwargs):
         hdulist.writeto(filename, clobber=True)
 
 
-def read_sky_map(filename, nest=False, distances=False, moc=False):
+def read_sky_map(filename, nest=False, distances=False, moc=False, **kwargs):
     """
     Read a LIGO/Virgo-type sky map and return a tuple of the HEALPix array
     and a dictionary of metadata from the header.
@@ -466,11 +466,12 @@ def read_sky_map(filename, nest=False, distances=False, moc=False):
     ...     m, meta = read_sky_map(f.name)
     ...     np.testing.assert_array_equal(m, hp.ring2nest(nside, ipix_nest))
     """
-    m = Table.read(filename, format='fits')
+    m = Table.read(filename, format='fits', **kwargs)
 
     # Remove some keys that we do not need
     for key in (
-            'PIXTYPE', 'EXTNAME', 'NSIDE', 'FIRSTPIX', 'LASTPIX', 'INDXSCHM'):
+            'PIXTYPE', 'EXTNAME', 'NSIDE', 'FIRSTPIX', 'LASTPIX', 'INDXSCHM',
+            'MOCORDER'):
         m.meta.pop(key, None)
 
     if m.meta.pop('COORDSYS', 'C') != 'C':
@@ -557,3 +558,6 @@ if __name__ == '__main__':
         runtime=21.5)
 
     print(read_sky_map('test.fits.gz'))
+
+from lalinference.bayestar.deprecation import warn
+warn('ligo.skymap.io.fits')
