@@ -47,7 +47,7 @@
 #include <gsl/gsl_errno.h>
 #include <gsl/gsl_spline.h>
 
-#define VARNAME_MAX 128
+#define VARNAME_MAX 40
 #define VARVALSTRINGSIZE_MAX 128
 
 #include <lal/LALStdlib.h>
@@ -59,8 +59,10 @@
 #include <lal/FindChirp.h>
 #include <lal/Window.h>
 #include <lal/LALString.h>
+#include <lal/StringInput.h>
 #include <lal/LALSimInspiral.h>
 #include <lal/LALSimInspiralWaveformCache.h>
+#include <lal/LALSimNeutronStar.h>
 #include <lal/LALHashTbl.h>
 
 #include <lal/SFTutils.h>
@@ -89,8 +91,6 @@
 /*LIB imports*/
 #include <lal/LALInferenceBurstRoutines.h>
 
-#define LALINFERENCE_HASHTABLE_SIZE 256 /* Max entries in hash table */
-
 //...other includes
 
 struct tagLALInferenceRunState;
@@ -452,6 +452,7 @@ typedef struct tagLALInferenceModel
   REAL8*                       ifo_SNRs; /** Array of single-IFO SNRs at *params* */
 
   REAL8                        fLow;   /** Start frequency for waveform generation */
+  REAL8                        fHigh;   /** End frequency for waveform generation */
   REAL8                        deltaT, deltaF;   /** Sampling rate information */
   INT4                         freqLength; /* Length of freq-domain buffer */
 
@@ -466,7 +467,8 @@ typedef struct tagLALInferenceModel
   REAL8Window                 *window;        /** A window */
   REAL8                        padding; /** The padding of the above window */
   struct tagLALInferenceROQModel *roq; /** ROQ data */
-  int roq_flag;
+  int roq_flag;               /** Is ROQ enabled */
+  LALSimNeutronStarFamily     *eos_fam; /** Neutron Star equation of state family */
 
 } LALInferenceModel;
 
@@ -617,7 +619,7 @@ tagLALInferenceRunState
 } LALInferenceRunState;
 
 
-#define DETNAMELEN 256
+#define DETNAMELEN 8
 
 /**
  * Structure to contain IFO data.
@@ -877,6 +879,12 @@ void LALInferenceQ2Eta(double q, double *eta);
 /** Convert from lambdaT, dLambdaT, and eta to lambda1 and lambda2. */
 void LALInferenceLambdaTsEta2Lambdas(REAL8 lambdaT, REAL8 dLambdaT, REAL8 eta, REAL8 *lambda1, REAL8 *lambda2);
 
+/** Calculate lambda1,2(m1,2|eos(logp1,gamma1,gamma2,gamma3)) */
+void LALInferenceLogp1GammasMasses2Lambdas(REAL8 logp1, REAL8 gamma1, REAL8 gamma2, REAL8 gamma3, REAL8 mass1, REAL8 mass2, REAL8 *lambda1, REAL8 *lambda2);
+
+/** Check for causality violation and mass conflict given masses and eos **/
+int LALInferenceEOSPhysicalCheck(LALInferenceVariables *params, ProcessParamsTable *commandLine);
+
 /**
  * The kD trees in LALInference are composed of cells.  Each cell
  * represents a rectangular region in parameter space, defined by
@@ -1151,11 +1159,19 @@ LALInferenceMCMCRunPhase* LALInferenceGetMCMCrunphase_ptrVariable(LALInferenceVa
 
 void LALInferenceSetMCMCrunphase_ptrVariable(LALInferenceVariables* vars,const char* name,LALInferenceMCMCRunPhase* value);
 
-void LALInferenceAddstringVariable(LALInferenceVariables * vars, const char * name, CHAR* value, LALInferenceParamVaryType vary);
+#ifdef SWIG   /* SWIG interface directives */
+SWIGLAL(OWNS_THIS_STRING(const CHAR*, value));
+#endif
 
-CHAR* LALInferenceGetstringVariable(LALInferenceVariables * vars, const char * name);
+void LALInferenceAddstringVariable(LALInferenceVariables * vars, const char * name, const CHAR* value, LALInferenceParamVaryType vary);
 
-void LALInferenceSetstringVariable(LALInferenceVariables* vars,const char* name,CHAR* value);
+const CHAR* LALInferenceGetstringVariable(LALInferenceVariables * vars, const char * name);
+
+void LALInferenceSetstringVariable(LALInferenceVariables* vars,const char* name, const CHAR* value);
+
+#ifdef SWIG   /* SWIG interface directives */
+SWIGLAL_CLEAR(OWNS_THIS_STRING(const CHAR*, value));
+#endif
 
 /**
  * Print spline calibration parameter names as tab-separated ASCII
