@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2016  Leo Singer
+ * Copyright (C) 2015-2017  Leo Singer
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include <gsl/gsl_sf_erf.h>
 #include <gsl/gsl_sf_exp.h>
 #include <gsl/gsl_cdf.h>
+#include <gsl/gsl_statistics.h>
 
 #include <chealpix.h>
 
@@ -80,24 +81,7 @@ double bayestar_distance_conditional_cdf(
 }
 
 
-static double ppf_f(double r, void *params)
-{
-    const double p = ((double*) params)[0];
-    const double mu = ((double *) params)[1];
-    const double norm = ((double *) params)[2];
-    return bayestar_distance_conditional_cdf(r, mu, 1, norm) - p;
-}
-
-
-static double ppf_df(double r, void *params)
-{
-    const double mu = ((double *) params)[1];
-    const double norm = ((double *) params)[2];
-    return bayestar_distance_conditional_pdf(r, mu, 1, norm);
-}
-
-
-static void ppf_fdf(double r, void *params, double *f, double *df)
+static void conditional_ppf_fdf(double r, void *params, double *f, double *df)
 {
     const double p = ((double*) params)[0];
     const double mu = ((double *) params)[1];
@@ -107,6 +91,22 @@ static void ppf_fdf(double r, void *params, double *f, double *df)
 }
 
 
+static double conditional_ppf_f(double r, void *params)
+{
+    double f, df;
+    conditional_ppf_fdf(r, params, &f, &df);
+    return f;
+}
+
+
+static double conditional_ppf_df(double r, void *params)
+{
+    double f, df;
+    conditional_ppf_fdf(r, params, &f, &df);
+    return df;
+}
+
+
 double bayestar_distance_conditional_ppf(
     double p, double mu, double sigma, double norm)
 {
@@ -133,7 +133,9 @@ double bayestar_distance_conditional_ppf(
     const gsl_root_fdfsolver_type *algo = gsl_root_fdfsolver_steffenson;
     char state[algo->size];
     gsl_root_fdfsolver solver = {algo, NULL, 0, state};
-    gsl_function_fdf fun = {ppf_f, ppf_df, ppf_fdf, &params};
+    gsl_function_fdf fun = {
+        conditional_ppf_f, conditional_ppf_df, conditional_ppf_fdf,
+        &params};
     gsl_root_fdfsolver_set(&solver, &fun, z);
 
     do
@@ -170,7 +172,8 @@ static void integrals(
 }
 
 
-static void fdf(double z, void *params, double *fval, double *dfval)
+static void moments_to_parameters_fdf(
+    double z, void *params, double *fval, double *dfval)
 {
     const double mean_std = *(double *)params;
     const double target = 1 / gsl_pow_2(mean_std) + 1;
@@ -181,18 +184,18 @@ static void fdf(double z, void *params, double *fval, double *dfval)
 }
 
 
-static double f(double z, void *params)
+static double moments_to_parameters_f(double z, void *params)
 {
     double fval, dfval;
-    fdf(z, params, &fval, &dfval);
+    moments_to_parameters_fdf(z, params, &fval, &dfval);
     return fval;
 }
 
 
-static double df(double z, void *params)
+static double moments_to_parameters_df(double z, void *params)
 {
     double fval, dfval;
-    fdf(z, params, &fval, &dfval);
+    moments_to_parameters_fdf(z, params, &fval, &dfval);
     return dfval;
 }
 
@@ -209,7 +212,11 @@ static int solve_z(double mean_std, double *result)
     const gsl_root_fdfsolver_type *algo = gsl_root_fdfsolver_steffenson;
     char state[algo->size];
     gsl_root_fdfsolver solver = {algo, NULL, 0, state};
-    gsl_function_fdf fun = {f, df, fdf, &mean_std};
+    gsl_function_fdf fun = {
+        moments_to_parameters_f,
+        moments_to_parameters_df,
+        moments_to_parameters_fdf,
+        &mean_std};
     gsl_root_fdfsolver_set(&solver, &fun, z);
 
     do
@@ -280,9 +287,42 @@ void bayestar_distance_parameters_to_moments(
 }
 
 
+static double bayestar_volume_render_inner(
+    double x, double y, double z, int axis0, int axis1, int axis2,
+    const double *R, long long nside, int nest, const double *prob, const
+    double *mu, const double *sigma, const double *norm)
+{
+    double ret;
+    double xyz[3];
+    xyz[axis0] = x;
+    xyz[axis1] = y;
+    xyz[axis2] = z;
+
+   /* Transform from screen-aligned cube to celestial coordinates before
+    * looking up pixel indices. */
+    double vec[3];
+    cblas_dgemv(
+        CblasRowMajor, CblasNoTrans, 3, 3, 1, R, 3, xyz, 1, 0, vec, 1);
+    int64_t ipix;
+    if (nest)
+        vec2pix_nest64(nside, vec, &ipix);
+    else
+        vec2pix_ring64(nside, vec, &ipix);
+    double r = sqrt(gsl_pow_2(x) + gsl_pow_2(y) + gsl_pow_2(z));
+
+    if (isfinite(mu[ipix]))
+        ret = gsl_sf_exp_mult(
+            -0.5 * gsl_pow_2((r - mu[ipix]) / sigma[ipix]),
+            prob[ipix] * norm[ipix] / sigma[ipix]);
+    else
+        ret = 0;
+    return ret;
+}
+
+
 double bayestar_volume_render(
     double x, double y, double max_distance, int axis0, int axis1,
-    const double *R, long nside, int nest,
+    const double *R, long long nside, int nest,
     const double *prob, const double *mu,
     const double *sigma, const double *norm)
 {
@@ -299,56 +339,161 @@ double bayestar_volume_render(
      * spatial origin to the plane of the screen. */
 
     /* Transverse distance from origin to point on screen */
-    double a = sqrt(gsl_pow_2(x) + gsl_pow_2(y));
+    const double a = sqrt(gsl_pow_2(x) + gsl_pow_2(y));
 
     /* Maximum value of theta (at edge of screen-aligned cube) */
-    double theta_max = atan2(max_distance, a);
-    double dtheta = 0.5 * M_PI / nside / 4;
+    const double theta_max = atan2(max_distance, a);
+    const double dtheta = 0.5 * M_PI / nside / 4;
 
-    /* Construct regular grid from -theta_max to +theta_max */
     double ret = 0;
-    for (double theta = -theta_max; theta <= theta_max; theta += dtheta)
+
+    /* Far from the center of the image, we integrate in theta so that we
+     * step through HEALPix pixels at an approximately uniform rate.
+     *
+     * In the central 10% of the image, we integrate in z to avoid the
+     * coordinate singularity in theta.
+     */
+    if (a / max_distance >= 5e-2)
     {
-        /* Differential z = a tan(theta),
-         * dz = dz/dtheta dtheta = a tan'(theta) dtheta = a sec^2(theta) dtheta,
-         * and dtheta = const */
-        double dz_dtheta = a / gsl_pow_2(cos(theta));
-        double z = a * tan(theta);
-        double xyz[3];
-        xyz[axis0] = x;
-        xyz[axis1] = y;
-        xyz[axis2] = z;
-
-       /* Transform from screen-aligned cube to celestial coordinates before
-        * looking up pixel indices. */
-        double vec[3];
-        cblas_dgemv(
-            CblasRowMajor, CblasNoTrans, 3, 3, 1, R, 3, xyz, 1, 0, vec, 1);
-        long ipix;
-        if (nest)
-            vec2pix_nest(nside, vec, &ipix);
-        else
-            vec2pix_ring(nside, vec, &ipix);
-        double r = sqrt(gsl_pow_2(x) + gsl_pow_2(y) + gsl_pow_2(z));
-
-        if (isfinite(mu[ipix]))
-            ret += gsl_sf_exp_mult(
-                -0.5 * gsl_pow_2((r - mu[ipix]) / sigma[ipix]),
-                prob[ipix] / sigma[ipix] * norm[ipix] * dz_dtheta);
+        /* Construct regular grid from -theta_max to +theta_max */
+        for (double theta = -theta_max; theta <= theta_max; theta += dtheta)
+        {
+            /* Differential z = a tan(theta),
+             * dz = dz/dtheta dtheta
+             *    = a tan'(theta) dtheta
+             *    = a sec^2(theta) dtheta,
+             * and dtheta = const */
+            const double dz_dtheta = a / gsl_pow_2(cos(theta));
+            const double z = a * tan(theta);
+            ret += bayestar_volume_render_inner(x, y, z, axis0, axis1, axis2,
+                R, nside, nest, prob, mu, sigma, norm) * dz_dtheta;
+        }
+        ret *= dtheta;
+    } else {
+        const double dz = max_distance * dtheta / theta_max;
+        for (double z = -max_distance; z <= max_distance; z += dz)
+        {
+            ret += bayestar_volume_render_inner(x, y, z, axis0, axis1, axis2,
+                R, nside, nest, prob, mu, sigma, norm);
+        }
+        ret *= dz;
     }
-
-    return ret / (M_SQRT2 * M_SQRTPI);
+    ret *= nside2npix64(nside) / (4 * M_PI * sqrt(2 * M_PI));
+    return ret;
 }
 
 
 double bayestar_distance_marginal_pdf(
-    double r, long npix,
+    double r, long long npix,
     const double *prob, const double *mu,
     const double *sigma, const double *norm)
 {
     double sum = 0;
-    for (long i = 0; i < npix; i ++)
+    #pragma omp parallel for reduction(+:sum)
+    for (long long i = 0; i < npix; i ++)
         sum += prob[i] * bayestar_distance_conditional_pdf(
             r, mu[i], sigma[i], norm[i]);
     return sum;
 }
+
+
+double bayestar_distance_marginal_cdf(
+    double r, long long npix,
+    const double *prob, const double *mu,
+    const double *sigma, const double *norm)
+{
+    double sum = 0;
+    #pragma omp parallel for reduction(+:sum)
+    for (long long i = 0; i < npix; i ++)
+        sum += prob[i] * bayestar_distance_conditional_cdf(
+            r, mu[i], sigma[i], norm[i]);
+    return sum;
+}
+
+
+typedef struct {
+    double p;
+    long long npix;
+    const double *prob;
+    const double *mu;
+    const double *sigma;
+    const double *norm;
+} marginal_ppf_params;
+
+
+static void marginal_ppf_fdf(double r, void *params, double *f, double *df)
+{
+    const marginal_ppf_params *p = (marginal_ppf_params *)params;
+    const double _f = bayestar_distance_marginal_cdf(
+        r, p->npix, p->prob, p->mu, p->sigma, p->norm);
+    const double _df = bayestar_distance_marginal_pdf(
+        r, p->npix, p->prob, p->mu, p->sigma, p->norm);
+    if (p->p > 0.5)
+    {
+        *f = log(1 - _f) - log(1 - p->p);
+        *df = -_df / (1 - _f);
+    } else {
+        *f = log(_f) - log(p->p);
+        *df = _df / _f;
+    }
+}
+
+
+static double marginal_ppf_f(double r, void *params)
+{
+    double f, df;
+    marginal_ppf_fdf(r, params, &f, &df);
+    return f;
+}
+
+
+static double marginal_ppf_df(double r, void *params)
+{
+    double f, df;
+    marginal_ppf_fdf(r, params, &f, &df);
+    return df;
+}
+
+
+double bayestar_distance_marginal_ppf(
+    double p, long long npix,
+    const double *prob, const double *mu,
+    const double *sigma, const double *norm)
+{
+    if (p <= 0)
+        return 0;
+    else if (p >= 1)
+        return GSL_POSINF;
+    else if (!isfinite(p))
+        return GSL_NAN;
+
+    /* Set up variables for tracking progress toward the solution. */
+    static const int max_iter = 50;
+    marginal_ppf_params params = {p, npix, prob, mu, sigma, norm};
+    int iter = 0;
+    const size_t max_ipix = gsl_stats_max_index(prob, 1, npix);
+    double r = bayestar_distance_conditional_ppf(
+        p, mu[max_ipix], sigma[max_ipix], norm[max_ipix]);
+    int status;
+
+    /* Set up solver (on stack). */
+    const gsl_root_fdfsolver_type *algo = gsl_root_fdfsolver_steffenson;
+    char state[algo->size];
+    gsl_root_fdfsolver solver = {algo, NULL, 0, state};
+    gsl_function_fdf fun = {
+        marginal_ppf_f, marginal_ppf_df, marginal_ppf_fdf,
+        &params};
+    gsl_root_fdfsolver_set(&solver, &fun, r);
+
+    do
+    {
+        const double rold = r;
+        status = gsl_root_fdfsolver_iterate(&solver);
+        r = gsl_root_fdfsolver_root(&solver);
+        status = gsl_root_test_delta (r, rold, 0, GSL_SQRT_DBL_EPSILON);
+        iter++;
+    } while (status == GSL_CONTINUE && iter < max_iter);
+    /* FIXME: do something with status? */
+
+    return r;
+}
